#!/usr/bin/env python3
# -*- encoding: UTF8 -*-

#######################################################################
#
#    Copyright (c) 2018 Stefan Helmert <stefan.helmert@t-online.de>
#
#######################################################################

from cryptdomainmgr.modules.common.cdmconfighelper import processConfig


# Default handling
# DEFAULT section overwritten by handler default configuration overwr. by explicit configuration

def interpreteValues(args):
    content = args['content']
    certconfig = args['config']
    certSecName = args['secname']
    handlerNames = content['handler'].split('/')
    if 1 < len(handlerNames):
        if 'letsencrypt' != handlerNames[1]:
            certconfig[certSecName]['caa'] = ''
    if 'keysize' in content:
        certconfig[certSecName]['keysize'] = int(content['keysize'])
    if 'extraflags' in content:
        certconfig[certSecName]['extraflags'] = [e for e in content['extraflags'].replace(' ', '').split(',') if len(e) > 0]

def interpreteConfig(cr, sh):
    return processConfig(cr, 'cert', postOp=interpreteValues, defaultConfig={'handler': 'dehydrated/letsencrypt', 'keysize': 4096, 'extraflags': ''})




#!/usr/bin/env python
# -*- encoding: UTF8 -*-

#######################################################################
#
#    Copyright (c) 2020 Stefan Helmert <stefan.helmert@t-online.de>
#
#######################################################################

import unittest

from cryptdomainmgr.cdmconfighandler import ConfigReader

def procConfig(cr):
    cr.open()
    cr.interprete(None)
    return cr


class TestCDMconfighandler(unittest.TestCase):
    def testEmptyConfig(self):
        cr = ConfigReader()
        procConfig(cr)
        self.assertEqual(cr.config, {})

    def testEmptyConfigContentList(self):
        cr = ConfigReader()
        cr.setContentList([])
        procConfig(cr)
        self.assertEqual(cr.config, {})

    def testEmptyConfigContentListEntry(self):
        cr = ConfigReader()
        cr.setContentList([''])
        procConfig(cr)
        self.assertEqual(cr.config, {})

    def testAddFilenames(self):
        cr = ConfigReader()
        cr.addFilenames(['a','b','c'])
        cr.addFilenames(['d','e','f'])
        self.assertEqual(cr.filenameList, ['a','b','c','d','e','f'])

    def testSetFilenames(self):
        cr = ConfigReader()
        cr.addFilenames(['a','b','c'])
        cr.addFilenames(['d','e','f'])
        cr.setFilenames(['g','h','i'])
        self.assertEqual(cr.filenameList, ['g','h','i'])

    def testSetContentList(self):
        cr = ConfigReader()
        cr.setContentList(['a','b','c'])
        cr.setContentList(['d','e','f'])
        cr.setContentList(['g','h\nr','i'])
        self.assertEqual(cr.contentList, ['g','h\nr','i'])


if "__main__" == __name__:
    unittest.main()






#!/usr/bin/env python3
# -*- encoding: UTF8 -*-

#######################################################################
#
#    Copyright (c) 2021 Stefan Helmert <stefan.helmert@t-online.de>
#
#######################################################################

import os
from shutil import copyfile
from subprocess import check_output, CalledProcessError
from simpleloggerplus import simpleloggerplus as log
from subprocess import check_output, CalledProcessError
from cryptdomainmgr.modules.common.cdmstatehelper import isReady
from cryptdomainmgr.modules.common.cdmfilehelper import makeDir
from cryptdomainmgr.modules.docker import dockerconfighelper as dch

import docker

defaultConfig = {'depends': 'dhparam, cert','dhparam': 'auto', 'cert': 'auto', 'dirint': 'auto', 'dirext': 'auto', 'dockersock': 'unix://var/run/docker.sock'}




def prepare(serviceConfig, serviceState, state):
    serviceState.setOpStateDone()

def rollover(serviceConfig, serviceState, state):
    serviceState.setOpStateWaiting()
    if not isReady(serviceConfig, state, ['dhparam', 'cert']):
        return
    serviceState.setOpStateRunning()
    certNames = serviceConfig['cert']


    if 'auto' == serviceConfig['dirext'] or 'auto' == serviceConfig['dirint']:
        client = docker.DockerClient(base_url=serviceConfig['dockersock'])

        containers = [c for c in client.containers.list() if c.attrs['Name'][1:] == serviceConfig['container']]
        if len(containers) == 0:
            log.error("container does not exist")
        container = containers[0]
        log.info("Updating traefik container: {}".format(container.attrs['Name']))
        destinations = [e.replace(' ','').split('=')[1] for e in container.attrs['Args'] if e.replace(' ','').split('=')[0] == '--providers.file.directory']
        if len(destinations) == 0:
            log.error("traefik container misses providers.file.directory argument")
        destination = destinations[0]


    traefikProvidersFileDirectory = dch.dockerPathMap('/'+serviceConfig['container'], destination)[0]

    #traefikProvidersFileDirectory = mount['Source'] if 'auto' == serviceConfig['dirext'] else serviceConfig['dirext'] #'./configuration'
    traefikConfigFilename = os.path.join(traefikProvidersFileDirectory,'files/configuration.toml')
    traefikCertDir = os.path.join(traefikProvidersFileDirectory,'certs')
    traefikProvidersFileDirectoryMount = os.path.normpath(destination) if 'auto' == serviceConfig['dirint'] else serviceConfig['dirint'] # '/configuration'
    traefikConfigFilenameMount = os.path.join(traefikProvidersFileDirectoryMount,'files/configuration.toml')
    traefikCertDirMount = os.path.join(traefikProvidersFileDirectoryMount,'certs')

    log.info(" -> Volume: {}:{}".format(traefikProvidersFileDirectory, traefikProvidersFileDirectoryMount))

    tcfc = ''
    for certName in certNames:
        certState = state.getSubstate('cert').getSubstate(certName).result
        certPath = os.path.join(traefikCertDir,certName,'cert.pem')
        keyPath = os.path.join(traefikCertDir,certName,'key.pem')
        certPathMount = os.path.join(traefikCertDirMount,certName,'cert.pem')
        keyPathMount = os.path.join(traefikCertDirMount,certName,'key.pem')
        makeDir(os.path.dirname(certPath))
        copyfile(certState['fullchainfile'], certPath)
        log.info('  {} -> {}'.format(certState['fullchainfile'], certPath))
        copyfile(certState['keyfile'], keyPath)
        log.info('  {} -> {}'.format(certState['keyfile'], keyPath))
        tcfc += '[[tls.certificates]]\n   certFile = "{}"\n   keyFile = "{}"\n\n'.format(certPathMount, keyPathMount)
        makeDir(os.path.dirname(traefikConfigFilename))
        with open(traefikConfigFilename,'w') as f:
            f.write(tcfc)
        # traefik reloads config and certs only if a random file is written in 
        # traefik's config root directory specified by:
        # --providers.file.directory = <config root directory>
        with open(os.path.join(traefikProvidersFileDirectory,'reloadtrigger'), 'w') as f:
            f.write('')

    log.info('  -> Traefik reload')
    serviceState.setOpStateDone()

def cleanup(serviceConfig, serviceState, state):
    serviceState.setOpStateDone()




#!/usr/bin/env python
# -*- encoding: UTF8 -*-

#######################################################################
#
#    Copyright (c) 2018 Stefan Helmert <stefan.helmert@t-online.de>
#
#######################################################################

import os
from subprocess import check_output, CalledProcessError
from simpleloggerplus import simpleloggerplus as log
from cryptdomainmgr.modules.common.cdmfilehelper import makeDir
import time

defaultConfig = {'filename': '/etc/myssl/dh2048.pem', 'keysize': 2048}

def prepare(dhparamConfig, dhparamState, statedir, dhparamSecName): 
    if 'openssl' != dhparamConfig['handler'].split('/')[0]:
        return
    keysize = 2048
    if 'keysize' in dhparamConfig:
        keysize = dhparamConfig['keysize']

    path = os.path.realpath(os.path.join(statedir, 'dhparams', dhparamSecName))
    makeDir(os.path.dirname(path))

    dhparamState.setOpStateRunning()
    try:
        rv = check_output(('openssl', 'dhparam', '-out', str(path), str(int(keysize))))
        log.info(rv)
    except CalledProcessError as e:
        log.error(e.output)
        time.sleep(1)
        raise(e)

    resDict = {}
    resDict['tmpfile'] = path

    if 'running' == dhparamState.opstate:
        dhparamState.registerResult(resDict)
    dhparamState.setOpStateDone()

    return rv





#!/usr/bin/env python
# -*- encoding: UTF8 -*-

#######################################################################
#
#    Copyright (c) 2020 Stefan Helmert <stefan.helmert@t-online.de>
#
#######################################################################

from cryptdomainmgr.modules.common.cdmconfighelper import applyDefault

def interpreteConfig(cr, sh):
    defaultTESTConfig = {'defaultx': '23', 'defaulty': '42'}
    testConfig = cr.getRawConfigOf('test')
    # apply general config defaults and the default section
    testConfig = applyDefault(testConfig, defaultTESTConfig) # must be here because following section depends on default values
    cr.updateConfig({'test': testConfig})
    return testConfig




#!/usr/bin/env python
# -*- encoding: UTF8 -*-

#######################################################################
#
#    Copyright (c) 2018 Stefan Helmert <stefan.helmert@t-online.de>
#
#######################################################################

from cryptdomainmgr.cdmconfighandler import ConfigReader
from cryptdomainmgr.cdmstatehandler import StateHandler
#from cryptdomainmgr.modules.common.cdmconfighelper import getStateDir
from simpleloggerplus import simpleloggerplus as log

def getNextPhase(currentPhase):
    if 'prepare' == currentPhase:
        return 'rollover'
    if 'rollover' == currentPhase:
        return 'cleanup'
    return 'prepare'

def getCurrentPhase(state, forcePhase='next'):
    if 'next' != str(forcePhase):
        return forcePhase
    if 'nextphase' in state.result:
        return state.result['nextphase']
    return 'prepare'

# maybe ToDo: readConfig() -> addConfig() + setConfig()
# run() with explicit parameter to overwrite config
# option to mix config files and config content in the priority list
# INFO about config files and content read
class ManagedDomain:
    def __init__(self):
        self.cr = ConfigReader()
        self.sh = StateHandler()

    def readConfig(self, confFiles=[], confContent=''):
        self.cr.setFilenames(confFiles)
        self.cr.setContentList(confContent)
        self.cr.open()
        if 'cdm' not in self.cr.cp:
            self.cr.cp['cdm'] = {}
        self.cr.interprete(self.sh)
        self.sh.registerConfig(self.cr.config['cdm'])

    def run(self, confFile=None, forcePhase='next', confContent=''):
        if confFile is not None or 0 < len(confContent):
            self.readConfig(confFile, confContent)
        self.sh.load()
        self.sh.delete() # --next starts with prepare, if failed
        self.sh.resetOpStateRecursive()
        currentPhase = getCurrentPhase(self.sh, forcePhase)
        log.info('Running phase: {}'.format(currentPhase))
        runPhase(self.cr, self.sh, currentPhase)
        nextphase = getNextPhase(currentPhase)
        self.sh.registerResult({'nextphase': nextphase})
        self.sh.save()


def runPhase(cr, sh, phase):
    sh.setOpStateRunning()
    handler = {secName: __import__('cryptdomainmgr.modules.'+str(secName)+'.main', fromlist=('cryptdomainmgr','modules')) for secName in cr.sections}
    for i in range(10):
        for k, v in handler.items():
            if not hasattr(v, phase):
                continue
            f = getattr(v, phase)
            f(cr.config, sh)
    sh.setOpStateDone()








#!/usr/bin/env python3
# -*- encoding: UTF8 -*-

#######################################################################
#
#    Copyright (c) 2018 Stefan Helmert <stefan.helmert@t-online.de>
#
#######################################################################

import os
import shutil
from subprocess import check_output, CalledProcessError
from simpleloggerplus import simpleloggerplus as log
from cryptdomainmgr.modules.common.cdmconfighelper import getStateDir
from cryptdomainmgr.modules.common.cdmfilehelper import makeDir

def createDomainAccessTable(config, domains):
    domainAccessTableStr = ""
    for d in domains:
        domainConfig = config['domain'][d]
        paramKeys = domainConfig['accessparams']
        domainAccessTableStr += ' ["{}"]="'.format(d) + '\\n'.join(["{}={}".format(k, domainConfig[k]) for k in paramKeys])+'"'
    return '({} )'.format(domainAccessTableStr)


def prepare(config, state):
    subState = state.getSubstate('cert')
    for certSecName, certConfig in config['cert'].items():
        if 'DEFAULT' == certSecName:
            continue
        if 'handler' not in certConfig:
            continue
        certState = subState.getSubstate(certSecName)
        if certState.isDone():
            continue
        domains = [k for k,v in config['domain'].items() if 'cert' in v and certSecName in v['cert']]
        log.info('Create certificate for section \"{}\"'.format(certSecName))
        log.info('  -> {}'.format(', '.join(domains)))
        log.debug(certConfig)
        domainAccessTable = createDomainAccessTable(config, domains)
        handlerNames = certConfig['handler'].split('/')
        handler = __import__('cryptdomainmgr.modules.cert.handler'+str(handlerNames[0]), fromlist=('cryptdomainmgr', 'modules','cert'))
        statedir = getStateDir(config, 'cert', certSecName)
        handler.prepare(certConfig, certState, statedir, domains, domainAccessTable)

def rollover(config, state):
    subState = state.getSubstate('cert')
    for certSecName, certConfig in config['cert'].items():
        if 'DEFAULT' == certSecName:
            continue
        if 'handler' not in certConfig:
            continue
        certState = subState.getSubstate(certSecName)
        if certState.isDone():
            continue
        log.info('Apply certificate for section \"{}\"'.format(certSecName))
        copyCert(certConfig, certState)
        certState.setOpStateDone()

def cleanup(config, state):
    subState = state.getSubstate('cert')
    for certSecName, certConfig in config['cert'].items():
        if 'DEFAULT' == certSecName:
            continue
        if 'handler' not in certConfig:
            continue
        certState = subState.getSubstate(certSecName)
        if certState.isDone():
            continue
        log.info('Cleanup certificate for section \"{}\"'.format(certSecName))
        delOldCert(certConfig, certState)
        certState.setOpStateDone()

def copyCert(certConfig, certState):
    src = os.path.dirname(certState.result['fullchainfile'])
    for name in certState.result['san']:
        dest = os.path.join(certConfig['destination'], name)
        log.info('  {} -> {}'.format(src, dest))
        try:
            makeDir(str(dest))
            for k in ['fullchainfile', 'chainfile', 'certfile', 'keyfile']:
                lnksrc = certState.result[k]
                lnkdst = os.path.join(os.path.dirname(lnksrc), os.readlink(lnksrc))
                shutil.copy2(lnkdst,os.path.join(dest, os.path.basename(lnksrc)))
        except CalledProcessError as e:
            log.error(e.output)
            raise(e)

def delOldCert(certConfig, certState):
    preserve = ['fullchainfile', 'certfile', 'keyfile', 'chainfile']
    preserveFiles = set([certState.result[e] for e in preserve])
    preserveFiles.update(set([os.path.realpath(e) for e in preserveFiles]))
    dirs = set([os.path.dirname(e) for e in preserveFiles])
    dirs = set([e for e in dirs if os.path.isdir(e)])
    dstFiles = set([os.path.join(certConfig['destination'], f, os.path.basename(e)) for e in preserveFiles for f in certState.result['san']])
    dstDirs = set([os.path.dirname(e) for e in dstFiles])
    dirs.update(dstDirs)
    allFiles = set([os.path.join(d, f) for d in dirs for f in os.listdir(d)])
    allFiles = set([e for e in allFiles if os.path.isfile(e)])
    removeFiles = allFiles - preserveFiles - dstFiles
    for e in removeFiles:
        log.info('  rm {}'.format(e))
        try:
            rv = check_output(('rm', str(e)))
        except CalledProcessError as e:
            log.error(e.output)
            raise(e)










#!/usr/bin/env python3
# -*- encoding: UTF8 -*-

import os

# VERSION
__version__ = "0.2.4-16"


if __name__ == '__main__':
    import git
    repo = git.Repo(search_parent_directories=False)
    version = '-'.join(repo.git.describe('--tags').split('-')[:2])
    if __version__ != version:
        print("Version in git repo changed!")
        print("  -> Change version in {} from {} to {}".format(__file__,
                __version__, version))
        __version__ = version
        with open(os.path.realpath(__file__), 'rt') as v:
            initfile = v.read()
            lines = initfile.splitlines()
            bg = [i for i, e in enumerate(lines) if 'VERSION' in e][0]
            lines[bg+1] = "__version__ = \"{}\"".format(__version__)
            initfile = '\n'.join(lines)
        with open(os.path.realpath(__file__), 'wt') as v:
            v.write(initfile)


#!/usr/bin/env python
# -*- encoding: UTF8 -*-

#######################################################################
#
#    Copyright (c) 2020 Stefan Helmert <stefan.helmert@t-online.de>
#
#######################################################################

import unittest
import os

from cryptdomainmgr.cdmstatehandler import StateHandler


class TestCDMstatehandler(unittest.TestCase):
    def testEmptyState(self):
        sh = StateHandler()
        self.assertEqual('uninitialized', sh.opstate)

    def testEmptyAllStates(self):
        sh = StateHandler()
        with self.subTest("initial state"):
            self.assertEqual('uninitialized', sh.opstate)
        sh.setOpStateWaiting()
        with self.subTest("wait state"):
            self.assertEqual('waiting', sh.opstate)
        sh.setOpStateRunning()
        with self.subTest("not done state"):
            self.assertFalse(sh.isDone())
        with self.subTest("run state"):
            self.assertEqual('running', sh.opstate)
        sh.setOpStateDone()
        with self.subTest("done state"):
            self.assertTrue(sh.isDone())

    def testRegisterResult(self):
        sh = StateHandler()
        sh.registerResult({'a': 42})
        self.assertEqual({'a': 42}, sh.result)

    def testRegisterConfig(self):
        sh = StateHandler()
        sh.registerConfig({'b': 23})
        self.assertEqual({'b': 23}, sh.config)

    def testDict(self):
        sh = StateHandler()
        with self.subTest("empty to dict"):
            self.assertEqual({'opstate': 'uninitialized', 'result': {}, 'config': {}, 'substate': {}}, sh.toDict())
        with self.subTest("empty to dict"):
            sh.fromDict({'opstate': 'done', 'result': {'b': 23}, 'config': {'c': 42}, 'substate': {}})
            self.assertEqual({'opstate': 'done', 'result': {'b': 23}, 'config': {'c': 42}, 'substate': {}}, sh.toDict())
        with self.subTest("from empty dict"):
            sh.fromDict({})
            self.assertEqual('uninitialized', sh.opstate)

    def testSubstate(self):
        sh = StateHandler()
        ssh1 = sh.getSubstate("mysubstate_1")
        ssh2 = sh.getSubstate("mysubstate_2")
        sssh21 = ssh2.getSubstate("mysubstate_21")
        sssh22 = ssh2.getSubstate("mysubstate_22")
        sssh22.setOpStateDone()
        sssh21.setOpStateWaiting()
        ssh2.setOpStateRunning()
        with self.subTest("set substates"):
            self.assertEqual({'opstate': 'uninitialized', 'result': {}, 'config': {},\
            'substate': {'mysubstate_1': {'opstate': 'uninitialized', 'result': {}, 'config': {}, 'substate': {}},\
                         'mysubstate_2': {'opstate': 'running', 'result': {}, 'config': {}, \
                                          'substate': {'mysubstate_21': {'opstate': 'waiting', 'result': {}, 'config': {}, 'substate': {}}, \
                             'mysubstate_22': {'opstate': 'done', 'result': {}, 'config': {}, 'substate': {}}}}}}, sh.toDict())
        with self.subTest("reset substates"):
            sh.resetOpStateRecursive()
            self.assertEqual({'opstate': 'uninitialized', 'result': {}, 'config': {},\
            'substate': {'mysubstate_1': {'opstate': 'uninitialized', 'result': {}, 'config': {}, 'substate': {}},\
                         'mysubstate_2': {'opstate': 'uninitialized', 'result': {}, 'config': {}, \
                                          'substate': {'mysubstate_21': {'opstate': 'uninitialized', 'result': {}, 'config': {}, 'substate': {}}, \
                             'mysubstate_22': {'opstate': 'uninitialized', 'result': {}, 'config': {}, 'substate': {}}}}}}, sh.toDict())

    def testSubstateSaveLoad(self):
        sh = StateHandler()
        ssh1 = sh.getSubstate("mysubstate_1")
        ssh2 = sh.getSubstate("mysubstate_2")
        sssh21 = ssh2.getSubstate("mysubstate_21")
        sssh22 = ssh2.getSubstate("mysubstate_22")
        sssh22.setOpStateDone()
        sssh21.setOpStateWaiting()
        ssh2.setOpStateRunning()
        sh.config = {'DEFAULT': {'statedir': '/tmp/test_cdmstatehandler'}}
        sh.save()
        ssh1.setOpStateDone()
        sh.save('/tmp/test_cdmstatehandler_2/state2.json')
        shLoad1 = StateHandler()
        shLoad1.load('/tmp/test_cdmstatehandler_2/state2.json')
        with self.subTest("set substates - load direkt"):
            self.assertEqual({'opstate': 'uninitialized', 'result': {},
                              'config': {'DEFAULT': {'statedir':
                                         '/tmp/test_cdmstatehandler'}},\
            'substate': {'mysubstate_1': {'opstate': 'done', 'result': {}, 'config': {}, 'substate': {}},\
                         'mysubstate_2': {'opstate': 'running', 'result': {}, 'config': {}, \
                                          'substate': {'mysubstate_21': {'opstate': 'waiting', 'result': {}, 'config': {}, 'substate': {}}, \
                             'mysubstate_22': {'opstate': 'done', 'result': {}, 'config': {}, 'substate': {}}}}}}, shLoad1.toDict())
        shLoad2 = StateHandler()
        shLoad2.config = {'DEFAULT': {'statedir': '/tmp/test_cdmstatehandler'}}
        shLoad2.load()
        with self.subTest("set substates - load via config"):
            self.assertEqual({'opstate': 'uninitialized', 'result': {},
                              'config': {'DEFAULT': {'statedir':
                                                     '/tmp/test_cdmstatehandler'}},\
            'substate': {'mysubstate_1': {'opstate': 'uninitialized', 'result': {}, 'config': {}, 'substate': {}},\
                         'mysubstate_2': {'opstate': 'running', 'result': {}, 'config': {}, \
                                          'substate': {'mysubstate_21': {'opstate': 'waiting', 'result': {}, 'config': {}, 'substate': {}}, \
                             'mysubstate_22': {'opstate': 'done', 'result': {}, 'config': {}, 'substate': {}}}}}}, shLoad2.toDict())
        with self.subTest("delete via config - before"):
            self.assertTrue(os.path.isfile('/tmp/test_cdmstatehandler/state.json'))
        sh.delete()
        with self.subTest("delete via config - after"):
            self.assertFalse(os.path.isfile('/tmp/test_cdmstatehandler/state.json'))
        with self.subTest("delete direct - before"):
            self.assertTrue(os.path.isfile('/tmp/test_cdmstatehandler_2/state2.json'))
        sh.delete('/tmp/test_cdmstatehandler_2/state2.json')
        with self.subTest("delete direct - after"):
            self.assertFalse(os.path.isfile('/tmp/test_cdmstatehandler_2/state2.json'))



#!/usr/bin/env python
# -*- encoding: UTF8 -*-

#######################################################################
#
#    Copyright (c) 2018 Stefan Helmert <stefan.helmert@t-online.de>
#
#######################################################################

from simpleloggerplus import simpleloggerplus as log
from cryptdomainmgr.modules.common.cdmconfighelper import getStateDir

def prepare(config, state):
    subState = state.getSubstate('dkim')
    for dkimSecName, dkimConfig in config['dkim'].items():
        if 'DEFAULT' == dkimSecName:
            continue
        if 'handler' not in dkimConfig:
            continue
        dkimState = subState.getSubstate(dkimSecName)
        if dkimState.isDone():
            continue
        log.info("Preparing DKIM key for dkim-section: \"{}\"".format(dkimSecName))
        handlerNames = dkimConfig['handler'].split('/')
        handler = __import__('cryptdomainmgr.modules.dkim.handler'+str(handlerNames[0]), fromlist=('cryptdomainmgr','modules','dkim'))
        statedir = getStateDir(config, 'dkim', dkimSecName)
        handler.prepare(dkimConfig, dkimState, statedir)

def rollover(config, state):
    subState = state.getSubstate('dkim')
    for dkimSecName, dkimConfig in config['dkim'].items():
        if 'DEFAULT' == dkimSecName:
            continue
        if 'handler' not in dkimConfig:
            continue
        dkimState = subState.getSubstate(dkimSecName)
        if dkimState.isDone():
            continue
        log.info("Applying DKIM key for dkim-section: \"{}\"".format(dkimSecName))
        handlerNames = dkimConfig['handler'].split('/')
        handler = __import__('cryptdomainmgr.modules.dkim.handler'+str(handlerNames[0]), fromlist=('cryptdomainmgr','modules','dkim'))
        handler.rollover(dkimConfig, dkimState)

def cleanup(config, state):
    subState = state.getSubstate('dkim')
    for dkimSecName, dkimConfig in config['dkim'].items():
        if 'DEFAULT' == dkimSecName:
            continue
        if 'handler' not in dkimConfig:
            continue
        dkimState = subState.getSubstate(dkimSecName)
        if dkimState.isDone():
            continue
        log.info("Cleanup DKIM key for dkim-section: \"{}\"".format(dkimSecName))
        handlerNames = dkimConfig['handler'].split('/')
        handler = __import__('cryptdomainmgr.modules.dkim.handler'+str(handlerNames[0]), fromlist=('cryptdomainmgr','modules','dkim'))
        handler.cleanup(dkimConfig, dkimState)





#!/usr/bin/env python3
# -*- encoding: UTF8 -*-

#######################################################################
#
#    Copyright (c) 2018 Stefan Helmert <stefan.helmert@t-online.de>
#
#######################################################################

from cryptdomainmgr.modules.common.cdmconfighelper import applyDefault
from simpleloggerplus import simpleloggerplus as log

def filterEntries(content, rrType):
    return {k: v for k, v in content.items() if rrType == k[:len(rrType)]}

def splitList(content):
    return {k: [e.strip() for e in v.split(',')] for k, v in content.items()}

def parseNestedEntry(key, value, default = [], keySplitPattern = '.', valueSplitPattern = ':'):
    if keySplitPattern == '':
        keyList = [key]
    else:
        keyList   = key.split(keySplitPattern)
    if valueSplitPattern == '':
        valueList = [value]
    else:
        valueList = value.split(valueSplitPattern)
    addList   = list(default)
    rv = {}
    subtractMode = False
    if '+' == keyList[-1][-1]:
        keyList[-1] = keyList[-1][:-1].strip()
    elif '-' == keyList[-1][-1]:
        keyList[-1] = keyList[-1][:-1].strip()
        subtractMode = True
    else:
        rv['delList'] = list(keyList)
    addList[:len(keyList)] = list(keyList)
    for i, e in enumerate(valueList):
        try:
            addList[-i] = e
        except IndexError:
            log.warn('Too many arguments in parameter!')
    if subtractMode is True:
        rv['delListWithContent'] = addList
    else:
        rv['addList'] = addList
    return rv

def list2dict(entry, hasContent, keys):
    entry.extend(['*'] * (len(keys) - len(entry)))
    rv = {key: entry[i] for i, key in enumerate(keys)}
    rv = {k: v for k, v in rv.items() if '*' != v}
    if hasContent is False:
        if keys[0] in rv:
            del rv[keys[0]]
    return rv

def list2set(entry, hasContent):
    if hasContent is False:
        return set(entry[1:])
    return set(entry)

def list2SPF(spfEntry, hasContent = True):
    if hasContent is False:
        return '*'
    return spfEntry[0]

def list2CAA(caaEntry, hasContent = True):
    return list2dict(caaEntry, hasContent, ['flag', 'url', 'tag'])

def list2SRV(srvEntry, hasContent = True):
    return list2dict(srvEntry, hasContent, ['server', 'service', 'proto', 'port', 'weight', 'prio'])

def list2MX(mxEntry, hasContent = True):
    return list2dict(mxEntry, hasContent, ['content', 'prio'])

def list2ip(ipEntry, hasContent = True):
    return list2dict(ipEntry, hasContent, ['content'])

def list2dkim(dkimEntry, hasContent = True):
    return list2dict(dkimEntry, hasContent, ['op', 'content'])

def list2tlsa(tlsaEntry, hasContent = True):
    return list2dict(tlsaEntry, hasContent, ['op', 'proto', 'port', 'matchingtype', 'selector', 'usage'])

def list2rrType(rrType, entry, hasContent = True):
    if 'mx' == rrType:
        return list2MX(entry, hasContent)
    elif 'srv' == rrType:
        return list2SRV(entry, hasContent)
    elif 'ip4' == rrType:
        return list2ip(entry, hasContent)
    elif 'ip6' == rrType:
        return list2ip(entry, hasContent)
    elif 'dkim' == rrType:
        return list2dkim(entry, hasContent)
    elif 'tlsa' == rrType:
        return list2tlsa(entry, hasContent)
    elif 'caa' == rrType:
        return list2CAA(entry, hasContent)
    elif 'spf' == rrType:
        return list2SPF(entry, hasContent)
    log.error('rrType not supported')
    assert TypeError('rrType not supported')

def interpreteRR(content, rrType = 'mx', defaultList = ['*', '10'], keySplitPattern = '.', valueSplitPattern = ':'):
    x = filterEntries(content, rrType)
    conf = splitList(x)
    parsedList = [parseNestedEntry(k, e, defaultList, keySplitPattern, valueSplitPattern) for k, v in conf.items() for e in v]
    aggrAdd = [list2rrType(rrType, e['addList']) for e in parsedList if 'addList' in e]
    aggrDel = [list2rrType(rrType, e['delList'], False) for e in parsedList if 'delList' in e]
    aggrDelWithContent = [list2rrType(rrType, e['delListWithContent']) for e in parsedList if 'delListWithContent' in e]
    aggrDel.extend(aggrDelWithContent)
    return {'{}AggrAdd'.format(rrType): aggrAdd, '{}AggrDel'.format(rrType): aggrDel}

def interpreteCAA(content):
    return interpreteRR(content, 'caa', ['*', '*', '*'], '.', None) # None = one or more spaces

def interpreteMX(content):
    return interpreteRR(content, 'mx', ['*', '10'])

def interpreteSRV(content):
    return interpreteRR(content, 'srv', ['*', 'http', 'tcp', '443', '0', '0'])

def interpreteA(content):
    return interpreteRR(content, 'ip4', ['*'])

def interpreteAAAA(content):
    return interpreteRR(content, 'ip6', ['*'], '', '') # '' = do not split

def interpreteDKIM(content):
    return interpreteRR(content, 'dkim', ['*', '*'])

def interpreteTLSA(content):
    return interpreteRR(content, 'tlsa', ['*', 'tcp', '*', '3', '0', '1'])

def interpreteDMARC(content):
    return interpreteDictRR(content, 'dmarc')

def interpreteSOA(content):
    return interpreteDictRR(content, 'soa')

def interpreteSPF(content):
    return interpreteSetRR(content, 'spf', ['*'])

def interpreteCert(content):
    if 'cert' not in content:
        return {}
    certNames = content['cert'].replace(' ','').split(',')
    return {'cert': certNames}


def interpreteHandler(content):
    if 'handler' not in content:
        return {}
    handlerNames = content['handler'].split('/')
    handler = __import__('cryptdomainmgr.modules.domain.handler{}'.format(handlerNames[0]), fromlist=('cryptdomainmgr', 'modules', 'domain'))
    return {'accessparams': handler.getAccessParams(content)}

def interpreteDictRR(content, rrType):
    if rrType in [k.split('.')[0] for k in content.keys()]:
        rrDict = {k.split('.')[1] if 1 < len(k.split('.')) else '' : v  for k, v in content.items() if rrType == k.split('.')[0]}
        return {rrType: rrDict}
    return {}

def interpreteSetRR(content, rrType, defaultList = ['*']):
    rrList = interpreteRR(content, rrType, defaultList, '', '')
    rrSet = {k: set([e for e in v if '' != e]) for k, v in rrList.items()}
    return rrSet

def interpreteConfig(cr, sh):
    domainconfigOrig = cr.getRawConfigOf('domain', True)
    domainconfig = applyDefault(domainconfigOrig) # must be here because following section depends on default values
    for domain, content in domainconfig.items():
        for f in ['A', 'AAAA', 'DKIM', 'TLSA', 'MX', 'SRV', 'CAA', 'DMARC', 'SOA', 'SPF', 'Handler', 'Cert']:
            domainconfig[domain].update((globals()['interprete{}'.format(f)](content)))
        depend = 'cert'
        dependSections = set(cr.getRawConfigOf(depend).keys())
        if depend in domainconfig[domain]:
            dependEntry = set(domainconfig[domain][depend])
            if 'auto' in dependEntry:
                dependEntry = dependEntry - 'auto' + dependSections
            doesNotExist = dependEntry - dependSections
            for missing in doesNotExist:
                log.warn("Section {}:{} referenced in domain:{} does not exist!".format(depend,missing,domain))
            domainconfig[domain][depend] = list(set(domainconfig[domain][depend]) - doesNotExist)
        addDKIMcont = set([e['content'] for e in domainconfig[domain]['dkimAggrAdd']])
        dkimSections = set(cr.getRawConfigOf('dkim').keys())
        doesNotExist = addDKIMcont - dkimSections
        for missing in doesNotExist:
            log.warn("Section dkim:{} referenced in domain:{} does not exist!".format(missing,domain))
        for e in domainconfig[domain]['dkimAggrAdd']:
            if e['content'] not in doesNotExist:
                del e


    log.debug(domainconfig)
    cr.updateConfig({'domain': domainconfig})
    return domainconfig







#!/usr/bin/env python
# -*- encoding: UTF8 -*-

#######################################################################
#
#    Copyright (c) 2018 Stefan Helmert <stefan.helmert@t-online.de>
#
#######################################################################

import configparser
import io
from simpleloggerplus import simpleloggerplus as log

class ConfigReader:
    def __init__(self):
        self.cp = configparser.ConfigParser()
        self.filenameList = []
        self.contentList = []
        self.config = {}

    def setContentList(self, contentList):
        if contentList is None:
            return
        if type(contentList) is str:
            contentList = [contentList]
        self.contentList = contentList

    def setFilenames(self, filenames):
        if filenames is None:
            return
        if type(filenames) is str:
            filenames = [filenames]
        self.filenameList = filenames

    def addFilenames(self, filenames):
        if filenames is None:
            return
        if type(filenames) is str:
            filenames = [filenames]
        self.filenameList.extend(filenames)

    def open(self):
        self.cp = configparser.ConfigParser()
        self.cp.read(self.filenameList)
        for e in self.contentList:
            tmp = io.BytesIO(e.encode('utf8'))
            self.cp.read_file(io.TextIOWrapper(tmp))

    def getRawConfigOf(self, getSection, domainOldStyle=False):
        return getConfigOf(getSection, self.cp, domainOldStyle)

    def updateConfig(self, config):
        self.config.update(config)

    def interprete(self, sh):
        self.sections = getSections(self.cp)
        log.info('Interpreting config sections')
        for secName in self.sections:
            log.info('  - {}'.format(secName))
            handler = __import__('cryptdomainmgr.modules.'+str(secName)+'.confighandler', fromlist=('cryptdomainmgr','modules'))
            handler.interpreteConfig(self, sh)
        log.debug(self.config)

def getSections(config):
    return set([k.split(':')[0] for k, v in config.items() if k != 'DEFAULT'])


def getConfigOf(getSection, config, domainOldStyle=False):
    resConfig = {}
    for name, content in config.items():
        section = name.split(':')
        if getSection != section[0]:
            if domainOldStyle is True:
                if '.' not in section[0]: # fallback if not domain:example.de but example.de
                    continue
            else:
                continue
        if 2 == len(section):
            secName = section[1]
        else:
            secName = 'DEFAULT'
            if domainOldStyle is True:
                if '.' in section[0]:
                    secName = section[0] # fallback if not domain:example.de but example.de
        resConfig[secName] = dict({str(k): str(v) for k, v in content.items()})
    return resConfig




#!/usr/bin/env python3
# -*- encoding: UTF8 -*-

#######################################################################
#
#    Copyright (c) 2018 Stefan Helmert <stefan.helmert@t-online.de>
#
#######################################################################

from cryptdomainmgr.modules.common.cdmconfighelper import processConfig
from cryptdomainmgr.modules.common.cdmconfighelper import applyDefault
from simpleloggerplus import simpleloggerplus as log


# Default handling
# DEFAULT section overwritten by handler default configuration overwr. by explicit configuration

def resolveAuto(config, serviceConfig, depends):
    if 'auto' in serviceConfig:
        log.debug("This will never be called!")
        if depends not in config:
            return []
        dependKeys = config[depends]
        serviceConfig.extend(dependKeys)
        serviceConfig = [e for e in serviceConfig if e != 'auto' if e != 'DEFAULT']
    return serviceConfig


def interpreteValues(args):
    depends = args['content']['depends'].replace(' ','').split(',')
    args['config'][args['secname']]['depends'] = set(depends) if 0 < len(depends[0]) else {}
    for depend in depends:
        if depend in args['content']:
            args['config'][args['secname']][depend] = resolveAuto(args['config'], args['content'][depend].replace(' ','').split(','), depend)



def readHandlerDefault(args):
    handlerName = ''
    if 'handler' not in args['content']:
        handlerName = 'auto'
    else:
        handlerName = args['content']['handler']
    if 'auto' == handlerName:
        handlerName = args['secname']
    if 'DEFAULT' == args['secname']:
        return
    handlerNames = handlerName.split('/')
    handler = __import__('cryptdomainmgr.modules.{}.handler{}'.format(str(args['module']), handlerNames[0]), fromlist=('cryptdomainmgr', 'modules', str(args['module'])))
    args['config'][args['secname']].update(handler.defaultConfig)
    args['config'][args['secname']].update({'handler': handlerName})


def interpreteConfig(cr, sh):
    return processConfig(cr, 'service', preOp=readHandlerDefault, postOp=interpreteValues, defaultConfig={'container': 'false', 'depends': ''})







#!/usr/bin/env python3
# -*- encoding: UTF8 -*-

#######################################################################
#
#    Copyright (c) 2021 Stefan Helmert <stefan.helmert@t-online.de>
#
#######################################################################

import subprocess as sp
import sys
from simpleloggerplus import simpleloggerplus as log

def runCmdGen(cmd, stderr=sp.STDOUT, env=None):
    if int(sys.version.split('.')[1]) < 7:
        proc = sp.Popen(cmd, stdout=sp.PIPE, stderr=stderr, shell=True,
                        universal_newlines=True, env=env)
    else:
        proc = sp.Popen(cmd, stdout=sp.PIPE, stderr=stderr, shell=True,
                        text=True, encoding='utf-8', env=env)
    for stdoutLine in iter(proc.stdout.readline, ""):
        yield stdoutLine
    proc.stdout.close()
    rc = proc.wait()
    if rc:
        raise sp.CalledProcessError(rc, cmd)

def runCmd(cmd, stderr=sp.STDOUT, env=None):
    log.info("RUN CMD: {}".format(cmd))
    stdout = ""
    try:
        for stdoutLine in runCmdGen(cmd, stderr, env):
            log.relog("    "+stdoutLine[:-1])
            stdout += stdoutLine
    except sp.CalledProcessError as e:
        raise sp.CalledProcessError(e.returncode, e.cmd, stdout)
    return stdout




#!/usr/bin/env python3
# -*- encoding: UTF8 -*-

#######################################################################
#
#    Copyright (c) 2018 Stefan Helmert <stefan.helmert@t-online.de>
#
#######################################################################


# returns also True if section/module not in config
def isReady(serviceConfig, state, sec):
    if type(sec) is list:
        for e in sec:
            if isReady(serviceConfig, state, e) is False:
                return False
        return True
    subState = state.getSubstate(sec)
    if sec not in serviceConfig:
        return True
    return 0 == len([0 for e in serviceConfig[sec] if not subState.getSubstate(e).isDone()])




#!/usr/bin/env python
# -*- encoding: UTF8 -*-

#######################################################################
#
#    Copyright (c) 2018 Stefan Helmert <stefan.helmert@t-online.de>
#
#######################################################################

from simpleloggerplus import simpleloggerplus as log
from subprocess import check_output, CalledProcessError
from cryptdomainmgr.modules.common.cdmstatehelper import isReady

defaultConfig = {'depends': 'dhparam, cert', 'cert': 'auto', 'dhparam': 'auto'}

def prepare(serviceConfig, serviceState, state):
    serviceState.setOpStateDone()

def rollover(serviceConfig, serviceState, state):
    serviceState.setOpStateWaiting()
    if not isReady(serviceConfig, state, ['cert', 'dhparam']):
        return
    serviceState.setOpStateRunning()
    log.info('  -> Dovecot reload')
    try:
        rv = check_output(('sudo', 'systemctl', 'start', 'dovecot'))
        rv = check_output(('sudo', 'systemctl', 'reload', 'dovecot'))
    except CalledProcessError as e:
        log.error(e.output)
        raise(e)
    serviceState.setOpStateDone()

def cleanup(serviceConfig, serviceState, state):
    serviceState.setOpStateDone()





#!/usr/bin/env python
# -*- encoding: UTF8 -*-

#######################################################################
#
#    Copyright (c) 2020 Stefan Helmert <stefan@entroserv.de>
#
#######################################################################

import unittest
import subprocess as sp


class TestMain(unittest.TestCase):
    def testMainHelp(self):
        stdout = sp.check_output("python3 -m cryptdomainmgr -h", shell=True)
        print(stdout[:5])
        self.assertEqual(b"usage", stdout[:5])

    def testMainCDMupdate(self):
        stdout = sp.check_output("python3 -m cryptdomainmgr --update \
                                 --config-content \
                                 '[cdm]    statedir=/tmp/test_cryptdomainmgr' \
                                 2>&1", shell=True)
        self.assertIn("update", stdout.decode())

    def testMainCDMupdateFile(self):
        stdout = sp.check_output("python3 -m cryptdomainmgr --update \
                                 test/testconfigfile.conf \
                                 2>&1", shell=True)
        self.assertIn("update", stdout.decode())

    def testMainCDMprepareFile(self):
        stdout = sp.check_output("python3 -m cryptdomainmgr --prepare \
                                 test/testconfigfile.conf \
                                 2>&1", shell=True)
        self.assertIn("prepare", stdout.decode())

    def testMainCDMrolloverFile(self):
        stdout = sp.check_output("python3 -m cryptdomainmgr --rollover \
                                 test/testconfigfile.conf \
                                 2>&1", shell=True)
        self.assertIn("rollover", stdout.decode())

    def testMainCDMcleanupFile(self):
        stdout = sp.check_output("python3 -m cryptdomainmgr --cleanup \
                                 test/testconfigfile.conf \
                                 2>&1", shell=True)
        self.assertIn("cleanup", stdout.decode())

    def testMainCDMnextFile(self):
        stdout = sp.check_output("python3 -m cryptdomainmgr \
                                 test/testconfigfile.conf \
                                 2>&1", shell=True)
        with self.subTest("prepare"):
            self.assertIn("prepare", stdout.decode())

        stdout = sp.check_output("python3 -m cryptdomainmgr \
                                 test/testconfigfile.conf \
                                 2>&1", shell=True)
        with self.subTest("rollover"):
            self.assertIn("rollover", stdout.decode())

        stdout = sp.check_output("python3 -m cryptdomainmgr \
                                 test/testconfigfile.conf \
                                 2>&1", shell=True)
        with self.subTest("cleanup"):
            self.assertIn("cleanup", stdout.decode())



#!/usr/bin/env python3
# -*- encoding: UTF8 -*-

#######################################################################
#
#    Copyright (c) 2018 Stefan Helmert <stefan.helmert@t-online.de>
#
#######################################################################

from simpleloggerplus import simpleloggerplus as log
from cryptdomainmgr.modules.common.cdmstatehelper import isReady


def update(config, state):
    subState = state.getSubstate('domain')
    for domainSecName, domainConfig in config['domain'].items():
        if 'DEFAULT' == domainSecName:
            continue
        if 'handler' not in domainConfig:
            continue
        domainState = subState.getSubstate(domainSecName)
        if domainState.isDone():
            continue
        log.info('Create resource records for section \"{}\"'.format(domainSecName))
        log.debug(domainConfig)
        caaAuto(domainConfig, config)
        handlerNames = domainConfig['handler'].split('/')
        handler = __import__('cryptdomainmgr.modules.domain.handler'+str(handlerNames[0]), fromlist=('cryptdomainmgr', 'modules','domain'))
        handler.update(domainConfig, domainState, domainSecName)


def prepare(config, state):
    subState = state.getSubstate('domain')
    for domainSecName, domainConfig in config['domain'].items():
        if 'DEFAULT' == domainSecName:
            continue
        if 'handler' not in domainConfig:
            continue
        domainState = subState.getSubstate(domainSecName)
        if domainState.isDone():
            continue
        log.info('Create resource records for section \"{}\"'.format(domainSecName))
        domainState.setOpStateWaiting()
        if not isReady(domainConfig, state, ['cert', 'dkim']):
            return
        domainState.setOpStateRunning()
        log.debug(domainConfig)
        handlerNames = domainConfig['handler'].split('/')
        handler = __import__('cryptdomainmgr.modules.domain.handler'+str(handlerNames[0]), fromlist=('cryptdomainmgr', 'modules','domain'))
        handler.prepare(domainConfig, domainState, domainSecName, state)

def rollover(config, state):
    subState = state.getSubstate('domain')
    for domainSecName, domainConfig in config['domain'].items():
        if 'DEFAULT' == domainSecName:
            continue
        if 'handler' not in domainConfig:
            continue
        domainState = subState.getSubstate(domainSecName)
        if domainState.isDone():
            continue
        log.info('Create resource records for section \"{}\"'.format(domainSecName))
        handlerNames = domainConfig['handler'].split('/')
        handler = __import__('cryptdomainmgr.modules.domain.handler'+str(handlerNames[0]), fromlist=('cryptdomainmgr', 'modules','domain'))
        handler.rollover(domainConfig, domainState, domainSecName, state)

def cleanup(config, state):
    subState = state.getSubstate('domain')
    for domainSecName, domainConfig in config['domain'].items():
        if 'DEFAULT' == domainSecName:
            continue
        if 'handler' not in domainConfig:
            continue
        domainState = subState.getSubstate(domainSecName)
        if domainState.isDone():
            continue
        log.info('Create resource records for section \"{}\"'.format(domainSecName))
        handlerNames = domainConfig['handler'].split('/')
        handler = __import__('cryptdomainmgr.modules.domain.handler'+str(handlerNames[0]), fromlist=('cryptdomainmgr', 'modules','domain'))
        handler.cleanup(domainConfig, domainState, domainSecName, state)


def caaAuto(domainConfig, config):
    if 'caa' not in domainConfig:
        return
    if 'auto' not in domainConfig['caa']:
        return
    domainConfig['caaAggrAdd'] = [e if e['flag'] != 'auto' else config['cert'][domainConfig['cert']]['caa'] for e in domainConfig['caaAggrAdd']]




#!/usr/bin/env python3
# -*- encoding: UTF8 -*-

#######################################################################
#
#    Copyright (c) 2020 Stefan Helmert <stefan.helmert@t-online.de>
#
#######################################################################

import os

testdomain = "test42.bahn.cf"
testns = "ns2.inwx.de"
tmpdir="/tmp/test_cryptdomainmgr"
testcertpath = os.path.join(tmpdir, "ssl")
testcertemail = "stefan.helmert@t-online.de"
testdh = "dh512"
dhfile = os.path.join(tmpdir,"myssl/dh512.pem")




#!/usr/bin/env python
# -*- encoding: UTF8 -*-

#######################################################################
#
#    Copyright (c) 2018 Stefan Helmert <stefan.helmert@t-online.de>
#
#######################################################################

from cryptdomainmgr.modules.common.cdmconfighelper import processConfig


# Default handling
# DEFAULT section overwritten by handler default configuration overwr. by explicit configuration


def interpreteValues(args):
    if 'keysize' in args['content']:
        args['config'][args['secname']]['keysize'] = int(args['content']['keysize'])

def interpreteConfig(cr, sh):
    return processConfig(cr, 'dhparam', postOp=interpreteValues, defaultConfig={'filename': '/etc/myssl/dhparam.pem', 'keysize': 2048})




#!/usr/bin/env python
# -*- encoding: UTF8 -*-

#######################################################################
#
#    Copyright (c) 2018 Stefan Helmert <stefan.helmert@t-online.de>
#
#######################################################################

import argparse
import re
from cryptdomainmgr.cdmcore import ManagedDomain

if '__main__' == __name__:
    parser = argparse.ArgumentParser(prog='cryptdomainmgr', description='Cryptdomainmgr handles interleaved certificate, tlsa, dkim renewal and domain update.')
    parser.add_argument('config_files', metavar='configfiles', type=str, nargs='*', default=[], help='list of configuration files')
    parser.add_argument('--next', dest='phase', action='store_const', const='next', help='Runs rollover if previous run was prepare or cleanup if rollover or prepare if cleanup/first run.') 
    parser.add_argument('--update', dest='phase', action='store_const', const='update', help='Running update sets all static records including A and AAAA regualrly not having an expire date. This should be run as very first time e. g. to upload CAA records.') 
    parser.add_argument('--prepare', dest='phase', action='store_const', const='prepare', help='Creates new certificates and dkim keys. Publishes new TLSA and DKIM records.') 
    parser.add_argument('--rollover', dest='phase', action='store_const', const='rollover', help='Applies new certificates and dkim keys.') 
    parser.add_argument('--cleanup', dest='phase', action='store_const', const='cleanup', help='Removes old unused records and files.') 
    parser.add_argument('--config-content', dest='configcontent', type=str, default='', help='configuration content as argument instead of configuration files')


    args = parser.parse_args()
    configcontent = re.compile(' {2,}').sub('\n', str(args.configcontent))
    #configcontent = str(args.configcontent).replace('  ', '\n')

    if args.phase is None:
        args.phase = 'next'

    mgr = ManagedDomain()
    mgr.run(args.config_files, args.phase, configcontent)








#!/usr/bin/env python
# -*- encoding: UTF8 -*-

#######################################################################
#
#    Copyright (c) 2020 Stefan Helmert <stefan.helmert@t-online.de>
#
#######################################################################


def prepare(config, state):
    subState = state.getSubstate('test')
    subState.setOpStateDone()

    for testSecName, testConfig in config['test'].items():
        sub2State = subState.getSubstate(testSecName)
        sub2State.setOpStateDone()




#!/usr/bin/env python3
# -*- encoding: UTF8 -*-

#######################################################################
#
#    Copyright (c) 2021 Stefan Helmert <stefan.helmert@t-online.de>
#
#######################################################################


import time
from cryptdomainmgr.modules.common.cdmstatehelper import isReady
from simpleloggerplus import simpleloggerplus as log

def postwait(config, state):
    #print(config)
    subState = state.getSubstate('cdm')
    if subState.isDone():
        return
    for cdmSecName, cdmConfig in config['cdm'].items():
        cdmState = subState.getSubstate(cdmSecName)
        if cdmState.isDone():
            continue
        if not isReady(cdmConfig, state, ['dhparam', 'cert', 'domain', 'dkim',
                                         'service']):
        #if not isReady(cdmConfig, state, ['cert']):
            return
        if 'postwait' in cdmConfig:
            T = cdmConfig['postwait']
            log.info('Wait {} s after run!'.format(T))
            time.sleep(int(T))
    subState.setOpStateDone()

def prepare(config, state):
    postwait(config, state)

def rollover(config, state):
    postwait(config, state)

def cleanup(config, state):
    postwait(config, state)




#!/usr/bin/env python3
# -*- encoding: UTF8 -*-

#######################################################################
#
#    Copyright (c) 2018 Stefan Helmert <stefan.helmert@t-online.de>
#
#######################################################################

from subprocess import check_output, CalledProcessError

def serviceAction(serviceName, action):
    return check_output(('sudo', 'systemctl', action, serviceName))


def reloadService(serviceName):
    return serviceAction(serviceName, 'reload')

def restartService(serviceName):
    return serviceAction(serviceName, 'restart')

def startService(serviceName):
    return serviceAction(serviceName, 'start')

def stopService(serviceName):
    return serviceAction(serviceName, 'stop')





#!/usr/bin/env python
# -*- encoding: UTF8 -*-

#######################################################################
#
#    Copyright (c) 2018 Stefan Helmert <stefan.helmert@t-online.de>
#
#######################################################################

import os
from simpleloggerplus import simpleloggerplus as log

def applyDefault(config, defaultConfig={}):
    default = dict(defaultConfig)
    if 'DEFAULT' in config:
        default.update(config['DEFAULT'])
    newconfig = {}
    for section, content in config.items():
        newconfig[section] = dict(default)
        newconfig[section].update(content)
    return newconfig

def getStateDir(config, section=None, handler=None):
    statedir = config['cdm']['DEFAULT']['statedir']
    statedir = os.path.join(statedir, 'modules')
    if section is not None:
        statedir = os.path.join(statedir, '{}'.format(section))
    if handler is not None:
        statedir = os.path.join(statedir, '{}'.format(handler))
    return statedir

def readHandlerDefault(args):
    if 'handler' in args['content']:
        log.debug('handler in content')
        handlerNames = args['content']['handler'].split('/')
        handler = __import__('cryptdomainmgr.modules.{}.handler{}'.format(str(args['module']), handlerNames[0]), fromlist=('cryptdomainmgr', 'modules', str(args['module'])))
        args['config'][args['secname']].update(handler.defaultConfig)

def processConfig(configReader, module, postOp=None, preOp=readHandlerDefault, defaultConfig={}):
    configOrig = configReader.getRawConfigOf(str(module))
    # apply general config defaults and the default section
    config = applyDefault(configOrig, defaultConfig) # must be here because following section depends on default values
    log.debug(config)
    for secName, content in config.items():
        if preOp is not None:
            preOp({'content': dict(content), 'module': str(module), 'config': config, 'secname': str(secName)})
        config[secName].update(configOrig[secName])
        if postOp is not None:
            postOp({'content': dict(content), 'module': str(module), 'config': config, 'secname': str(secName)})
    log.debug(config)
    configReader.updateConfig({str(module): config})
    return config




#!/usr/bin/env python3
# -*- encoding: UTF8 -*-

#######################################################################
#
#    Copyright (c) 2020 Stefan Helmert <stefan.helmert@t-online.de>
#
#######################################################################

import unittest
import copy

from cryptdomainmgr.cdmcore import getNextPhase, getCurrentPhase, StateHandler, ConfigReader, runPhase, ManagedDomain

def procConfig(cr):
    cr.open()
    cr.interprete(None)
    return cr

class StateTest:
    def __init__(self):
        self.result = {"nextphase": "myphase"}

class TestCDMcore(unittest.TestCase):
    def testGetNextPhase(self):
        self.assertEqual("rollover", getNextPhase("prepare"))
        self.assertEqual("cleanup", getNextPhase("rollover"))
        self.assertEqual("prepare", getNextPhase("cleanup"))
        self.assertEqual("prepare", getNextPhase("update"))
        self.assertEqual("prepare", getNextPhase("blub"))

    def testGetCurrentPhase(self):
        s = StateTest()
        self.assertEqual("myphase", getCurrentPhase(s))
        self.assertEqual("forcedphase", getCurrentPhase(s, "forcedphase"))
        del s.result["nextphase"]
        self.assertEqual("prepare", getCurrentPhase(s))

    def testRunPhase(self):
        sh = StateHandler()
        cr = ConfigReader()
        cr.setContentList(['[test]\nbla = blub\n[test:mytestname]\nbla = blubber'])
        procConfig(cr)
        runPhase(cr, sh, "prepare")
        with self.subTest("<opstate>"):
            self.assertEqual("done", sh.opstate)
        with self.subTest("test/<opstate>"):
            self.assertEqual("done", sh.getSubstate('test').opstate)
        with self.subTest("test/DEFAULT/<opstate>"):
            self.assertEqual("done", sh.getSubstate('test').getSubstate('DEFAULT').opstate)
        with self.subTest("test/mytestname/<opstate>"):
            self.assertEqual("done", sh.getSubstate('test').getSubstate('mytestname').opstate)

    def testManagedDomain(self):
        md = ManagedDomain()
        #md.readConfig(content="[cdm]\nstatedir={}\n[test:mytest]".format(tmpStateFile.name))
        md.run(confContent="[cdm]\nstatedir=/tmp/test_cryptdomainmgr\n[test:mytest]", forcePhase="update")
        sh = md.sh
        with self.subTest("<config>"):
            res = copy.deepcopy(sh.config)
            res['DEFAULT']['depends'] = set(res['DEFAULT']['depends'])
            self.assertEqual({'DEFAULT': {'depends': {'dhparam', 'domain',
                                                      'dkim', 'service',
                                                      'cert'}, 'statedir':
                                          '/tmp/test_cryptdomainmgr'}}, res)
        with self.subTest("<result> (first)"):
            self.assertEqual({'nextphase': 'prepare'}, sh.result)
        md = ManagedDomain()
        md.run(confContent="[cdm]\nstatedir=/tmp/test_cryptdomainmgr\n[test]\n[test:mytestname]")
        sh = md.sh
        with self.subTest("<opstate>"):
            self.assertEqual("done", sh.opstate)
        with self.subTest("test/<opstate>"):
            self.assertEqual("done", sh.getSubstate('test').opstate)
        with self.subTest("test/DEFAULT/<opstate>"):
            self.assertEqual("done", sh.getSubstate('test').getSubstate('DEFAULT').opstate)
        with self.subTest("test/mytestname/<opstate>"):
            self.assertEqual("done", sh.getSubstate('test').getSubstate('mytestname').opstate)
        with self.subTest("<result> (second)"):
            self.assertEqual({'nextphase': 'rollover'}, sh.result)
        md = ManagedDomain()
        md.run(confContent="[cdm]\nstatedir=/tmp/test_cryptdomainmgr\n[test:mytest]")
        sh = md.sh
        with self.subTest("<result> (third)"):
            self.assertEqual({'nextphase': 'cleanup'}, sh.result)
        md = ManagedDomain()
        md.run(confContent="[cdm]\nstatedir=/tmp/test_cryptdomainmgr\n[test:mytest]")
        sh = md.sh
        with self.subTest("<result> (fourth)"):
            self.assertEqual({'nextphase': 'prepare'}, sh.result)

    def testManagedDomainReadConfig(self):
        md = ManagedDomain()
        md.readConfig(confContent="[cdm]\nstatedir=/tmp/test_cryptdomainmgr\n[test]\n[test:mytestname]")
        #md.run(confContent="[cdm]\nstatedir=/tmp/test_cryptdomainmgr\n[test]\n[test:mytestname]")
        md.run(forcePhase='prepare')
        sh = md.sh
        with self.subTest("<opstate>"):
            self.assertEqual("done", sh.opstate)
        with self.subTest("test/<opstate>"):
            self.assertEqual("done", sh.getSubstate('test').opstate)
        with self.subTest("test/DEFAULT/<opstate>"):
            self.assertEqual("done", sh.getSubstate('test').getSubstate('DEFAULT').opstate)
        with self.subTest("test/mytestname/<opstate>"):
            self.assertEqual("done", sh.getSubstate('test').getSubstate('mytestname').opstate)

    def testManagedDomainReadConfig2(self):
        md = ManagedDomain()
        md.readConfig(confContent="  [cdm]  \n  statedir  =  /tmp/test_cryptdomainmgr  \n  [test]  \n  [test:mytestname] ")
        md.run(forcePhase='prepare')
        sh = md.sh
        with self.subTest("<opstate>"):
            self.assertEqual("done", sh.opstate)
        with self.subTest("test/<opstate>"):
            self.assertEqual("done", sh.getSubstate('test').opstate)
        with self.subTest("test/DEFAULT/<opstate>"):
            self.assertEqual("done", sh.getSubstate('test').getSubstate('DEFAULT').opstate)
        with self.subTest("test/mytestname/<opstate>"):
            self.assertEqual("done", sh.getSubstate('test').getSubstate('mytestname').opstate)

if "__main__" == __name__:
    unittest.main()







#!/usr/bin/env python
# -*- encoding: UTF8 -*-

#######################################################################
#
#    Copyright (c) 2018 Stefan Helmert <stefan.helmert@t-online.de>
#
#######################################################################

from cryptdomainmgr.modules.common.cdmconfighelper import processConfig


# Default handling
# DEFAULT section overwritten by handler default configuration overwr. by explicit configuration

def readHandlerDefault(args):
    if not 'keybasename' in args['content']:
        args['content']['keybasename'] = str(args['secname'])
    if 'handler' in args['content']:
        handlerNames = args['content']['handler'].split('/')
        handler = __import__('cryptdomainmgr.modules.{}.handler{}'.format(str(args['module']), handlerNames[0]), fromlist=('cryptdomainmgr', 'modules', str(args['module'])))
        args['config'][args['secname']].update(handler.defaultConfig)

def interpreteConfig(cr, sh):
    return processConfig(cr, 'dkim', preOp=readHandlerDefault, defaultConfig={'keysize': 2048})





#!/usr/bin/env python3

import os
import docker



def dockerMyContainerID():
    with open('/proc/self/cgroup', 'r') as f:
        cgroup = {e.split(':')[0]:e.split(':') for e in f.read().splitlines()}
        if 3 > len(cgroup['1']):
            return ''
        if 'docker' != cgroup['1'][2].split('/')[1]:
            return ''
        return cgroup['1'][2].split('/')[2]

def dockerPathMap(targetContainer, targetDirectory, dockersock='', sourceContainer=''):
    if '' == sourceContainer:
        sourceContainer = dockerMyContainerID()
    if '' == dockersock:
        client = docker.from_env()
    else:
        client = docker.DockerClient(base_url=dockersock)
    cAttrsbyName = {c.attrs['Name']: c.attrs for c in client.containers.list()}
    cAttrsbyId = {c.id: c.attrs for c in client.containers.list()}
    if targetContainer in cAttrsbyName.keys():
        targetContAttrs = cAttrsbyName[targetContainer]
    elif targetContainer in cAttrsbyId.keys():
        targetContAttrs = cAttrsbyId[targetContainer]
    if '' != sourceContainer:
        if sourceContainer in cAttrsbyName.keys():
            sourceContAttrs = cAttrsbyName[sourceContainer]
        elif sourceContainer in cAttrsbyId.keys():
            sourceContAttrs = cAttrsbyId[sourceContainer]
        sourceMap = {e['Destination']: e['Source'] for e in sourceContAttrs['Mounts'] if True == e['RW']}
    else:
        sourceMap = {'/': '/'}
    targetMap = {e['Destination']: e['Source'] for e in targetContAttrs['Mounts']}
    targetMounts = {os.path.normpath(d): os.path.normpath(s) for d, s in targetMap.items() if os.path.normpath(d) == os.path.commonpath([os.path.normpath(d),os.path.normpath(targetDirectory)])}
    targetSources = [os.path.join(s,os.path.relpath(os.path.normpath(targetDirectory), d)) for d, s in targetMounts.items()]
    sourceDestinations = [os.path.join(d,os.path.relpath(os.path.normpath(ts),s))  for ts in targetSources for d, s in sourceMap.items() if os.path.normpath(s) == os.path.commonpath([os.path.normpath(s),os.path.normpath(ts)])]
    return sourceDestinations





#!/usr/bin/env python3

import setuptools
import cryptdomainmgr


with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="cryptdomainmgr",
    version=cryptdomainmgr.__version__,
    author="Stefan Helmert",
    author_email="stefan.helmert@t-online.de",
    description="Software managing certificate, dkim and domain updates automagically.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://www.entroserv.de/offene-software/cryptdomainmgr",
    packages=setuptools.find_packages(exclude=['test*']),
    include_package_data=True,
    install_requires=[
        "Jinja2",
        "simpleloggerplus",
        "pyOpenSSL",
        "dnsuptools",
        "parse",
        "configparser"
    ],
    license = 'https://www.fsf.org/licensing/licenses/agpl-3.0.html',
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU Affero General Public License v3",
        "Operating System :: OS Independent",
    ],
)




#!/usr/bin/env python
# -*- encoding: UTF8 -*-

#######################################################################
#
#    Copyright (c) 2018 Stefan Helmert <stefan.helmert@t-online.de>
#
#######################################################################

import os
from subprocess import check_output, CalledProcessError
from simpleloggerplus import simpleloggerplus as log
from cryptdomainmgr.modules.common.cdmconfighelper import getStateDir

def prepare(config, state):
    subState = state.getSubstate('dhparam')
    for dhparamSecName, dhparamConfig in config['dhparam'].items():
        if 'DEFAULT' == dhparamSecName:
            continue
        if 'handler' not in dhparamConfig:
            continue
        dhparamState = subState.getSubstate(dhparamSecName)
        if dhparamState.isDone():
            continue
        log.info('Create dhparams for section \"{}\"'.format(dhparamSecName))
        log.debug(dhparamConfig)
        handlerNames = dhparamConfig['handler'].split('/')
        handler = __import__('cryptdomainmgr.modules.dhparam.handler'+str(handlerNames[0]), fromlist=('cryptdomainmgr', 'modules','dhparam'))
        statedir = getStateDir(config, 'dhparam', dhparamSecName)
        handler.prepare(dhparamConfig, dhparamState, statedir, dhparamSecName) 

def rollover(config, state):
    subState = state.getSubstate('dhparam')
    for dhparamSecName, dhparamConfig in config['dhparam'].items():
        if 'DEFAULT' == dhparamSecName:
            continue
        if 'handler' not in dhparamConfig:
            continue
        dhparamState = subState.getSubstate(dhparamSecName)
        if dhparamState.isDone():
            continue
        log.info('Apply dhparams for section \"{}\"'.format(dhparamSecName))
        copyDH(dhparamConfig, dhparamState)
        dhparamState.setOpStateDone()

def cleanup(config, state):
    subState = state.getSubstate('dhparam')
    for dhparamSecName, dhparamConfig in config['dhparam'].items():
        if 'DEFAULT' == dhparamSecName:
            continue
        if 'handler' not in dhparamConfig:
            continue
        dhparamState = subState.getSubstate(dhparamSecName)
        if dhparamState.isDone():
            continue
        log.info('Cleanup dhparams for section \"{}\"'.format(dhparamSecName))
        dhparamState.setOpStateDone()

def copyDH(dhparamConfig, dhparamState):
    if 'tmpfile' not in dhparamState.result:
        return
    src = os.path.realpath(dhparamState.result['tmpfile'])
    dest = os.path.realpath(dhparamConfig['filename'])
    try:
        os.makedirs(os.path.dirname(dest))
    except:
        pass
    log.info('  {} -> {}'.format(src, dest))
    try:
        rv = check_output(('cp', '-rfLT', str(src), str(dest)))
    except CalledProcessError as e:
        log.error(e.output)
        raise(e)










#!/usr/bin/env python3
# -*- encoding: UTF8 -*-

#######################################################################
#
#    Copyright (c) 2018 Stefan Helmert <stefan.helmert@t-online.de>
#
#######################################################################

from simpleloggerplus import simpleloggerplus as log

def prepare(config, state):
    subState = state.getSubstate('service')
    for serviceSecName, serviceConfig in config['service'].items():
        if 'DEFAULT' == serviceSecName:
            continue
        if 'handler' not in serviceConfig:
            continue
        serviceState = subState.getSubstate(serviceSecName)
        if serviceState.isDone():
            continue
        log.info('Prepare service for section \"{}\"'.format(serviceSecName))
        log.debug(serviceConfig)
        handlerNames = serviceConfig['handler'].split('/')
        handler = __import__('cryptdomainmgr.modules.service.handler'+str(handlerNames[0]), fromlist=('cryptdomainmgr', 'modules','service'))
        handler.prepare(serviceConfig, serviceState, state)

def rollover(config, state):
    subState = state.getSubstate('service')
    for serviceSecName, serviceConfig in config['service'].items():
        if 'DEFAULT' == serviceSecName:
            continue
        if 'handler' not in serviceConfig:
            continue
        serviceState = subState.getSubstate(serviceSecName)
        if serviceState.isDone():
            continue
        log.info('Rollover service for section \"{}\"'.format(serviceSecName))
        log.debug(serviceConfig)
        handlerNames = serviceConfig['handler'].split('/')
        handler = __import__('cryptdomainmgr.modules.service.handler'+str(handlerNames[0]), fromlist=('cryptdomainmgr', 'modules','service'))
        handler.rollover(serviceConfig, serviceState, state)

def cleanup(config, state):
    subState = state.getSubstate('service')
    for serviceSecName, serviceConfig in config['service'].items():
        if 'DEFAULT' == serviceSecName:
            continue
        if 'handler' not in serviceConfig:
            continue
        serviceState = subState.getSubstate(serviceSecName)
        if serviceState.isDone():
            continue
        log.info('Cleanup service for section \"{}\"'.format(serviceSecName))
        log.debug(serviceConfig)
        handlerNames = serviceConfig['handler'].split('/')
        handler = __import__('cryptdomainmgr.modules.service.handler'+str(handlerNames[0]), fromlist=('cryptdomainmgr', 'modules','service'))
        handler.cleanup(serviceConfig, serviceState, state)





#!/usr/bin/env python3
# -*- encoding: UTF8 -*-

#######################################################################
#
#    Copyright (c) 2018 Stefan Helmert <stefan.helmert@t-online.de>
#
#######################################################################

import os
from subprocess import check_output, CalledProcessError, STDOUT
from simpleloggerplus import simpleloggerplus as log
from cryptdomainmgr.modules.common.cdmfilehelper import makeDir
from cryptdomainmgr.modules.common.cdmprochelper import runCmd
import time

defaultConfig = {'keysize': 4096, 'extraflags': '', 'caa': {'url': 'letsencrypt.org', 'flag': '0', 'tag': 'issue'}}

def prepare(certConfig, certState, statedir, domainList, domainAccessTable):
    if 'dehydrated' != certConfig['handler'].split('/')[0]:
        return
    if 0 == len(domainList):
        return
    email = certConfig['email']
    keysize = 4096
    if 'keysize' in certConfig:
        keysize = certConfig['keysize']
    if 'extraflags' in certConfig:
        extraFlags = certConfig['extraflags']
    extraFlags = [e if '-' == e[0] else '--' + e for e in extraFlags]
    if '--staging' in extraFlags:
        ca = "https://acme-staging-v02.api.letsencrypt.org/directory"
        extraFlags.remove('--staging')
        if 'staging' in extraFlags:
            extraFlags.remove('staging')
    elif 'ca' in certConfig:
        ca = certConfig['ca']
    else:
        ca = "https://acme-v02.api.letsencrypt.org/directory"

    makeDir(os.path.realpath(statedir))
    confFilename = os.path.normpath(os.path.join(statedir, 'dehydrated.conf'))
    confFile = open(confFilename, 'w')
    confFile.write('CA={}\n'.format(str(ca)))
    confFile.write('CONTACT_EMAIL={}\n'.format(str(email)))
    confFile.write('KEYSIZE={}\n'.format(int(keysize)))
    confFile.write('CERTDIR={}\n'.format(os.path.join(statedir, 'certs')))
    confFile.write('\n')
    confFile.close()


    here = os.path.dirname(os.path.realpath(__file__))
    args = [os.path.join(here, 'dehydrated/dehydrated'), '-f', confFilename, '--accept-terms', '-c', '-t', 'dns-01', '-k', os.path.join(here, 'hook.sh')]
    log.debug(extraFlags)
    args.extend(extraFlags)
    for d in domainList:
        args.extend(['-d', str(d)])
    log.debug(args)
    certState.setOpStateRunning()

    i = 0
    while True:
        try:
            log.info('Starting DNS-01 authentication')
            rv = runCmd(' '.join(args), stderr=STDOUT, env=dict(os.environ, DOMAINACCESSTABLE=domainAccessTable, STATEDIR=statedir, WAITSEC="{}".format(3**i)))
            break
        except CalledProcessError as e:
            if 'ERROR: Lock file' in e.output:
                lockFilename = e.output.decode().split('ERROR: Lock file \'')[-1].split('\' present, aborting')[0]
                log.warn('Lock file from imcomplete run found: {}'.format(lockFilename))
                log.warn('  -> Removing')
                os.remove(lockFilename)
            elif 'Incorrect TXT record' in e.output:
                log.info('  -> Invalid DNS-01 challenge, maybe due to DNS caching interval. Trying to wait longer!')
                i += 1
                log.info('  -> Will wait {} s to give challenge time to propagate DNS cache.'.format(3**i))
            elif 'NXDOMAIN' in e.output:
                log.info('  -> Missing DNS-01 challenge, maybe due to DNS caching interval. Trying to wait longer!')
                i += 1
                log.info('  -> Will wait {} s to give challenge time to propagate DNS cache.'.format(3**i))
            else:
                raise(e)
            if 9 == i:
                raise(e)

    res = []
    rv = rv.splitlines()
    for s, e in enumerate(rv):
        if '---- DEPLOYMENTRESULT ----' == e[:len('---- DEPLOYMENTRESULT ----')]:
            break
    for i, e in enumerate(rv[s+1:]):
        if '---- END DEPLOYMENTRESULT ----' == e[:len('---- END DEPLOYMENTRESULT ----')]:
            break
        res.append(e)
    resDict = {e.split('=')[0].lower(): e.split('=')[1] for e in res}
    resDict['san'] = list(domainList)

    if 'running' == certState.opstate:
        certState.registerResult(resDict)
    certState.setOpStateDone()

    return rv





#!/usr/bin/env python
# -*- encoding: UTF8 -*-

#######################################################################
#
#    Copyright (c) 2018 Stefan Helmert <stefan.helmert@t-online.de>
#
#######################################################################

from dnsuptools import dnsuptools
from simpleloggerplus import simpleloggerplus as log
from OpenSSL import crypto

def getAccessParams(domainConfig):
    if 'dnsuptools/inwx' == domainConfig['handler']:
        return ['handler', 'user', 'passwd']

def update(domainConfig, domainState, domainSecName):
    handlers = domainConfig['handler'].split('/')
    if 'dnsuptools' != handlers[0]:
        return
    dnsup = dnsuptools.DNSUpTools()
    dnsup.setHandler(handlers[1])
    dnsup.handler.setUserDict({'default': domainConfig['user'], domainSecName: domainConfig['user']})
    dnsup.handler.setPasswdDict({'default': domainConfig['passwd'], domainSecName: domainConfig['passwd']})

    domainState.setOpStateRunning()

    setACME(domainConfig, domainState, domainSecName, dnsup)
    setCAA(domainConfig, domainState, domainSecName, dnsup)
    setSOA(domainConfig, domainState, domainSecName, dnsup)
    setSPF(domainConfig, domainState, domainSecName, dnsup)
    setADSP(domainConfig, domainState, domainSecName, dnsup)
    setDMARCentries(domainConfig, domainState, domainSecName, dnsup)
    setSRV(domainConfig, domainState, domainSecName, dnsup)
    setIPs(domainConfig, domainState, domainSecName, dnsup)
    setMX(domainConfig, domainState, domainSecName, dnsup)

    domainState.setOpStateDone()

    return

def prepare(domainConfig, domainState, domainSecName, state):
    handlers = domainConfig['handler'].split('/')
    if 'dnsuptools' != handlers[0]:
        return
    dnsup = dnsuptools.DNSUpTools()
    dnsup.setHandler(handlers[1])
    dnsup.handler.setUserDict({'default': domainConfig['user'], domainSecName: domainConfig['user']})
    dnsup.handler.setPasswdDict({'default': domainConfig['passwd'], domainSecName: domainConfig['passwd']})


    domainState.setOpStateWaiting()

    domainState.setOpStateRunning()

    tlsaReady = addTLSA(domainConfig, domainState, domainSecName, dnsup, state)
    dkimReady = addDKIM(domainConfig, domainState, domainSecName, dnsup, state)
    if tlsaReady and dkimReady:
        domainState.setOpStateDone()

    return

def rollover(domainConfig, domainState, domainSecName, state):
    handlers = domainConfig['handler'].split('/')
    if 'dnsuptools' != handlers[0]:
        return

    domainState.setOpStateWaiting()

    domainState.setOpStateRunning()

    domainState.setOpStateDone()

    return

def cleanup(domainConfig, domainState, domainSecName, state):
    handlers = domainConfig['handler'].split('/')
    if 'dnsuptools' != handlers[0]:
        return
    dnsup = dnsuptools.DNSUpTools()
    dnsup.setHandler(handlers[1])
    dnsup.handler.setUserDict({'default': domainConfig['user'], domainSecName: domainConfig['user']})
    dnsup.handler.setPasswdDict({'default': domainConfig['passwd'], domainSecName: domainConfig['passwd']})

    domainState.setOpStateWaiting()

    domainState.setOpStateRunning()
    tlsaReady = delTLSA(domainConfig, domainState, domainSecName, dnsup, state)
    dkimReady = delDKIM(domainConfig, domainState, domainSecName, dnsup, state)
    if tlsaReady and dkimReady:
        domainState.setOpStateDone()

    return

def getCertSAN(filename):
    certFile = open(filename, 'rt').read()
    cert = crypto.load_certificate(crypto.FILETYPE_PEM, certFile)
    san = [cert.get_extension(i).get_data().split(b'\x82')[1:] for i in range(cert.get_extension_count()) if b'subjectAltName' == cert.get_extension(i).get_short_name()][0]
    san = [e[1:] for e in san]
    return san

#def getFullchain(state, domainContent):
#    certState = state.getSubstate('cert').getSubstate(domainContent['cert'][0])
#    return certState.result['fullchainfile']

def getFullchain(state, certSec):
    certState = state.getSubstate('cert').getSubstate(certSec)
    return certState.result['fullchainfile']

def getDKIMkeys(state, domainContent):
    if type(domainContent) is list:
        return [getDKIMkeys(state, e) for e in domainContent]
    dkimState = state.getSubstate('dkim').getSubstate(domainContent)
    return dkimState.result

def isReady(state, domainContent, sec):
    secState = state.getSubstate(sec).getSubstate(domainContent[sec][0])
    return secState.isDone()

def isCertReady(state, certSec):
    secState = state.getSubstate('cert').getSubstate(certSec)
    return secState.isDone()

# not used, but:
# maybe ToDo: use this function to equalize setTLSA and setDKIM implementation  
def isDKIMReady(state, dkimSec):
    secState = state.getSubstate('dkim').getSubstate(dkimSec)
    return secState.isDone()

def setSPF(domainConfig, domainState, domainSecName, dnsup):
    rrState = domainState.getSubstate('setspf')
    if rrState.isDone():
        return
    rrState.setOpStateRunning()
    if 'spfAggrDel' in domainConfig and 'spfAggrAdd' in domainConfig:
        dnsup.setSPFentry(domainSecName, domainConfig['spfAggrAdd'], domainConfig['spfAggrDel'])
    elif 'spfAggrDel' in domainConfig:
        dnsup.setSPFentry(domainSecName, {}, domainConfig['spfAggrDel'])
    elif 'spfAggrAdd' in domainConfig:
        dnsup.setSPFentry(domainSecName, domainConfig['spfAggrAdd'], {})
    rrState.setOpStateDone()

def setDMARCentries(domainConfig, domainState, domainSecName, dnsup):
    rrState = domainState.getSubstate('setdmarcentries')
    if rrState.isDone():
        return
    rrState.setOpStateRunning()
    if 'dmarc' in domainConfig:
        # following line means configuration:
        # dmarc = 
        # this deletes all entries not specified
        # if no configuration like:
        # dmarc = 
        # is specified, so only given dmarc subparameters are overwritten
        # all this is managed by dnsuptools:
        dnsup.setDMARCentry(domainSecName, domainConfig['dmarc'])
    rrState.setOpStateDone()

def setADSP(domainConfig, domainState, domainSecName, dnsup):
    rrState = domainState.getSubstate('setadsp')
    if rrState.isDone():
        return
    rrState.setOpStateRunning()
    if 'adsp' in domainConfig:
        dnsup.setADSP(domainSecName, domainConfig['adsp'])
    rrState.setOpStateDone()

def setACME(domainConfig, domainState, domainSecName, dnsup):
    rrState = domainState.getSubstate('setacme')
    if rrState.isDone():
        return
    rrState.setOpStateRunning()
    if 'acme' in domainConfig:
        dnsup.setACME(domainSecName, domainConfig['acme'])
    rrState.setOpStateDone()

def setSOA(domainConfig, domainState, domainSecName, dnsup):
    rrState = domainState.getSubstate('setsoa')
    if rrState.isDone():
        return
    rrState.setOpStateRunning()
    if 'soa' in domainConfig:
        dnsup.setSOAentry(domainSecName, domainConfig['soa'])
    rrState.setOpStateDone()

def addSRV(domainConfig, domainState, domainSecName, dnsup):
    rrState = domainState.getSubstate('addsrv')
    if rrState.isDone():
        return
    rrState.setOpStateRunning()
    if 'srvAggrAdd' in domainConfig:
        dnsup.addSRV(domainSecName, domainConfig['srvAggrAdd'])
    rrState.setOpStateDone()

def delSRV(domainConfig, domainState, domainSecName, dnsup):
    rrState = domainState.getSubstate('delsrv')
    if rrState.isDone():
        return
    rrState.setOpStateRunning()
    if 'srvAggrDel' in domainConfig:
        dnsup.delSRV(domainSecName, domainConfig['srvAggrDel'], domainConfig['srvAggrAdd'])
    rrState.setOpStateDone()

def setSRV(domainConfig, domainState, domainSecName, dnsup):
    addSRV(domainConfig, domainState, domainSecName, dnsup)
    delSRV(domainConfig, domainState, domainSecName, dnsup)

def addCAA(domainConfig, domainState, domainSecName, dnsup):
    rrState = domainState.getSubstate('addcaa')
    if rrState.isDone():
        return
    rrState.setOpStateRunning()
    if 'caaAggrAdd' in domainConfig:
        dnsup.addCAA(domainSecName, domainConfig['caaAggrAdd'])
    rrState.setOpStateDone()

def delCAA(domainConfig, domainState, domainSecName, dnsup):
    rrState = domainState.getSubstate('delcaa')
    if rrState.isDone():
        return
    rrState.setOpStateRunning()
    if 'caaAggrDel' in domainConfig:
        dnsup.delCAA(domainSecName, domainConfig['caaAggrDel'], domainConfig['caaAggrAdd'])
    rrState.setOpStateDone()

def setCAA(domainConfig, domainState, domainSecName, dnsup):
    addCAA(domainConfig, domainState, domainSecName, dnsup)
    delCAA(domainConfig, domainState, domainSecName, dnsup)

def delIPs(domainConfig, domainState, domainSecName, dnsup):
    rrState = domainState.getSubstate('delips')
    if rrState.isDone():
        return
    rrState.setOpStateRunning()
    if 'ip4AggrDel' in domainConfig:
        dnsup.delA(domainSecName, [e['content'] if 'content' in e else '*' for e in domainConfig['ip4AggrDel']], [e['content']  for e in domainConfig['ip4AggrAdd']])
    if 'ip6AggrDel' in domainConfig:
        dnsup.delAAAA(domainSecName, [e['content'] if 'content' in e else '*' for e in domainConfig['ip6AggrDel']], [e['content']  for e in domainConfig['ip6AggrAdd']])
    rrState.setOpStateDone()

def addIPs(domainConfig, domainState, domainSecName, dnsup):
    rrState = domainState.getSubstate('addips')
    if rrState.isDone():
        return
    rrState.setOpStateRunning()
    if 'ip4AggrAdd' in domainConfig:
        dnsup.addA(domainSecName, [e['content']  for e in domainConfig['ip4AggrAdd']])
    if 'ip6AggrAdd' in domainConfig:
        dnsup.addAAAA(domainSecName, [e['content']  for e in domainConfig['ip6AggrAdd']])
    rrState.setOpStateDone()
    return

def setIPs(domainConfig, domainState, domainSecName, dnsup):
    addIPs(domainConfig, domainState, domainSecName, dnsup)
    delIPs(domainConfig, domainState, domainSecName, dnsup)

def addMX(domainConfig, domainState, domainSecName, dnsup):
    rrState = domainState.getSubstate('addmx')
    if rrState.isDone():
        return
    rrState.setOpStateRunning()
    if 'mxAggrAdd' in domainConfig:
        dnsup.addMX(domainSecName, domainConfig['mxAggrAdd'])
    rrState.setOpStateDone()

def delMX(domainConfig, domainState, domainSecName, dnsup):
    rrState = domainState.getSubstate('delmx')
    if rrState.isDone():
        return
    rrState.setOpStateRunning()
    if 'mxAggrDel' in domainConfig:
        dnsup.delMX(domainSecName, domainConfig['mxAggrDel'], domainConfig['mxAggrAdd'])
    rrState.setOpStateDone()

def setMX(domainConfig, domainState, domainSecName, dnsup):
    addMX(domainConfig, domainState, domainSecName, dnsup)
    delMX(domainConfig, domainState, domainSecName, dnsup)



def addTLSA(domainConfig, domainState, domainSecName, dnsup, state):
    return setTLSA(domainConfig, domainState, domainSecName, dnsup, state, add=True, delete=False)

def delTLSA(domainConfig, domainState, domainSecName, dnsup, state):
    return setTLSA(domainConfig, domainState, domainSecName, dnsup, state, add=False, delete=True)

def setTLSA(domainConfig, domainState, domainSecName, dnsup, state, add=True, delete=True):
    rrState = domainState.getSubstate('settlsa')
    if rrState.isDone():
        return True
    if 'cert' in domainConfig and ('tlsaAggrAdd' in domainConfig or 'tlsaAggrDel' in domainConfig):
        rrState.setOpStateWaiting()
        tlsaPres = []
        for certSec in domainConfig['cert']:
            if not isCertReady(state, certSec):
                return False
            rrState.setOpStateRunning()
            cert = getFullchain(state, certSec)
            if cert is None:
                log.info('not deploying TLSA record for {} (no certificate)'.format(domainSecName))
            else:
                log.info('deploying TLSA record for {} (certificate found)'.format(domainSecName))
                sanList = getCertSAN(cert)
                log.info('  -> found certificate: {} for: {}'.format(cert, b', '.join(sanList)))
                if domainSecName.encode() not in sanList:
                    log.error('{} not in certificate {}'.format(domainSecName, cert))
                tlsaAdd = [dict(e, filename=cert) for e in domainConfig['tlsaAggrAdd'] if 'op' in e if 'auto' == e['op']]
                tlsaPres.extend(tlsaAdd)
                if add is True:
                    dnsup.addTLSA(domainSecName, tlsaAdd)
        log.info('removing old TLSA record for {}'.format(domainSecName))
        if delete is True:
            tlsaDel = domainConfig['tlsaAggrDel']
            dnsup.delTLSA(domainSecName, tlsaDel, tlsaPres)
    rrState.setOpStateDone()
    return True



def addDKIM(domainConfig, domainState, domainSecName, dnsup, state):
    return setDKIM(domainConfig, domainState, domainSecName, dnsup, state, add=True, delete=False)

def delDKIM(domainConfig, domainState, domainSecName, dnsup, state):
    return setDKIM(domainConfig, domainState, domainSecName, dnsup, state, add=False, delete=True)

def setDKIM(domainConfig, domainState, domainSecName, dnsup, state, add=True, delete=True):
    rrState = domainState.getSubstate('adddkim')
    if rrState.isDone():
        return True
    if 'dkimAggrAdd' in domainConfig or 'dkimAggrDel' in domainConfig:
        rrState.setOpStateWaiting()
        dkimReady = [1 for e in domainConfig['dkimAggrAdd'] if 'op' in e if 'auto' == e['op'] if not state.getSubstate('dkim').getSubstate(e['content']).isDone()]
        if not 0 == len(dkimReady):
            return False
        rrState.setOpStateRunning()
        dkimAdd = [{k: v for k, v in getDKIMkeys(state, e['content']).items() if k in ['v', 'k', 'p', 'keyname']} for e in domainConfig['dkimAggrAdd'] if 'op' in e if 'auto' == e['op']]
        if add is True:
            dnsup.addDKIM(domainSecName, dkimAdd)
        if delete is True:
            dkimDel = [{k: v for k, v in getDKIMkeys(state, e['content']).items() if k in ['v', 'k', 'p', 'keyname']} if 'content' in e else {} for e in domainConfig['dkimAggrDel']]
            dnsup.delDKIM(domainSecName, dkimDel, dkimAdd)
    rrState.setOpStateDone()
    return True







#!/usr/bin/env python3
# -*- encoding: UTF8 -*-

#######################################################################
#
#    Copyright (c) 2018 Stefan Helmert <stefan.helmert@t-online.de>
#
#######################################################################

from cryptdomainmgr.modules.common.cdmconfighelper import processConfig
from cryptdomainmgr.modules.common.cdmconfighelper import applyDefault

def resolveAuto(serviceConfig, dependSections):
    if 'auto' in serviceConfig:
        serviceConfig.extend(dependSections)
    serviceConfig = [e for e in serviceConfig if e != 'auto' and e !='DEFAULT']
    serviceConfig = list(set(serviceConfig))
    return serviceConfig


def interpreteValues(cr, args):
    depends = args['content']['depends'].replace(' ','').split(',')
    args['config'][args['secname']]['depends'] = list(set(depends)) #if 0 < len(depends[0]) else []
    for depend in depends:
        dependSections = cr.getRawConfigOf(depend).keys()
        if depend in args['content']:
            dependVals = args['content'][depend].replace(' ','').split(',')
            dependsResolved = resolveAuto(dependVals,dependSections)
            args['config'][args['secname']][depend] = dependsResolved
            if len(dependsResolved) > len(dependSections):
                log.warn("Dependency in config entry does not exist!")



def interpreteConfig(cr, sh):
    cnf =  processConfig(cr, 'cdm', preOp=None, postOp=lambda x:  interpreteValues(cr, x),
                         defaultConfig={'statedir': '/var/cryptdomainmgr',
                                        'depends': 'cert, domain, dkim, \
                                        dhparam, service'})
    return cnf


#def interpreteConfig(cr, sh):
#    defaultCDMConfig = {'statedir': '/var/cryptdomainmgr'}
#    cdmConfig = cr.getRawConfigOf('cdm')
#    # apply general config defaults and the default section
#    cdmConfig = applyDefault(cdmConfig, defaultCDMConfig) # must be here because following section depends on default values
#    cr.updateConfig({'cdm': cdmConfig['DEFAULT']})
#    return cdmConfig
#





#!/usr/bin/env python3
# -*- encoding: UTF8 -*-

#######################################################################
#
#    Copyright (c) 2018 Stefan Helmert <stefan.helmert@t-online.de>
#
#######################################################################

import json
import os
from .modules.common.cdmfilehelper import makeDir

def getDictDefault(dictVar, key, defaultVal):
    if key in dictVar:
        return dictVar[key]
    else:
        return defaultVal


class StateHandler:
    def __init__(self):
        self.opstate = ''
        self.result = {}
        self.config = {}
        self.substate = {}
        self.setOpStateUninitialized()

    def clear(self):
        self.opstate = ''
        self.result = {}
        self.config = {}
        self.substate = {}
        self.setOpStateUninitialized()


    def setOpState(self, opState):
        self.opstate = str(opState)

    def resetOpStateRecursive(self):
        self.setOpStateUninitialized()
        for k, v in self.substate.items():
            v.resetOpStateRecursive()

    # initial state
    def setOpStateUninitialized(self):
        self.setOpState("uninitialized")

    # waiting for results produced by other modules/handlers
    def setOpStateWaiting(self):
        self.setOpState("waiting")

    # module/handler is doing work
    def setOpStateRunning(self):
        self.setOpState("running")

    # module/handler is ready, results are written
    def setOpStateDone(self):
        self.setOpState("done")

    def isDone(self):
        return 'done' == self.opstate

    def isSubDone(self):
        return 0 == len([0 for k, v in self.substate.items() if not v.isDone()])


    # should be the output after handler/module run is done
    # example: certificate source = /etc/dehydrated/certs/exampel.domain
    def registerResult(self, result):
        self.result = result

    # should be the output known before module run
    # exmaple: certificate = letsencrypt -> CAA[auto] = letsencrypt
    def registerConfig(self, config):
        self.config = config

    def registerSubstate(self, sectionName):
        if str(sectionName) not in self.substate:
            self.substate[str(sectionName)] = StateHandler()

    def getSubstate(self, sectionName):
        self.registerSubstate(sectionName)
        return self.substate[str(sectionName)]

    def printAll(self, indent=0):
        print('{} opstate = {}'.format(indent * ' ', self.opstate))
        print('{} result  = {}'.format(indent * ' ', self.result))
        print('{} config  = {}'.format(indent * ' ', self.config))
        for k, v in self.substate.items():
            print('{} substate = {}'.format(indent * ' ', k))
            v.printAll(indent+1)

    def toDict(self):
        return {'opstate': self.opstate, 'result': self.result, 'config': self.config, 'substate': {k: v.toDict() for k, v in self.substate.items()}}

    def fromDict(self, stateDict):
        self.opstate = getDictDefault(stateDict, 'opstate', 'uninitialized')
        self.result = getDictDefault(stateDict, 'result', {})
        self.config = getDictDefault(stateDict, 'config', {})
        if 'substate' not in stateDict:
            self.substate = {}
            return
        for k, v in stateDict['substate'].items():
            self.substate[k] = StateHandler()
            self.substate[k].fromDict(v)

    def save(self, filename=None):
        if filename is None:
            filename = os.path.join(self.config['DEFAULT']['statedir'], 'state.json')
        makeDir(os.path.dirname(filename))
        with open(filename, 'w') as jsonfile:
            json.dump(self.toDict(), jsonfile)

    def delete(self, filename=None):
        if filename is None:
            filename = os.path.join(self.config['DEFAULT']['statedir'], 'state.json')
        if not os.path.isfile(filename):
            return
        os.remove(filename)

    def load(self, filename=None):
        if filename is None:
            filename = os.path.join(self.config['DEFAULT']['statedir'], 'state.json')
        if not os.path.isfile(filename):
            return
        with open(filename, 'r') as jsonfile:
            self.fromDict(json.load(jsonfile))





#!/usr/bin/env python
# -*- encoding: UTF8 -*-

#######################################################################
#
#    Copyright (c) 2018 Stefan Helmert <stefan.helmert@t-online.de>
#
#######################################################################

import os
from subprocess import check_output, CalledProcessError
from simpleloggerplus import simpleloggerplus as log
from jinja2 import Template
import time
from cryptdomainmgr.modules.common.cdmfilehelper import makeDir

here = os.path.dirname(os.path.realpath(__file__))
defaultConfig = {'keysize': 2048, 'keybasename': 'key', 'keylocation': '/var/lib/rspamd/dkim', 'keyname': 'dkim.key', 'signingconfdestinationfile': '/etc/rspamd/local.d/dkim_signing.conf', 'signingconftemplatefile': os.path.join(here, 'dkim_signing_template.conf')}

def prepare(dkimConfig, dkimState, statedir):
    if 'rspamd' == dkimConfig['handler'].split('/')[0]:
        res = createDKIM(dkimConfig['keysize'], os.path.join(statedir, 'key', 'dkim.key'))
        keySelector = createSelector(dkimConfig['keybasename'])
        u = createConf(dkimConfig['signingconftemplatefile'], os.path.join(statedir, 'conf', 'dkim.conf'), os.path.join(dkimConfig['keylocation'], dkimConfig['keyname']), keySelector)
        res.update(u)
        dkimState.registerResult(res)
        dkimState.setOpStateDone()

def rollover(dkimConfig, dkimState):
    if 'rspamd' == dkimConfig['handler'].split('/')[0]:
        log.info('using new dkim key, moving new config file')
        copyDKIM(dkimConfig, dkimState)
        copyConf(dkimConfig, dkimState)
        dkimState.setOpStateDone()

def cleanup(dkimConfig, dkimState):
    if 'rspamd' == dkimConfig['handler'].split('/')[0]:
        dkimState.setOpStateDone()

def copyDKIM(dkimConfig, dkimState):
    src = dkimState.result['keyfile']
    dest = os.path.join(dkimConfig['keylocation'], dkimConfig['keyname'])
    log.info('  {} -> {}'.format(src, dest))
    try:
        rv = check_output(('mkdir', '-p', os.path.dirname(str(dest))))
    except CalledProcessError as e:
        log.error("Failed to create directory path for {}".format(str(dest)))
    try:
        rv = check_output(('cp', '-rfLT', str(src), str(dest)))
    except CalledProcessError as e:
        log.error("Failed to copy file")
        log.error("  {} -> {}".format(str(src), str(dest)))
    try:
        rv = check_output(('chown', '_rspamd:_rspamd', str(dest)))
    except CalledProcessError as e:
        log.error("Failed to change ownership of {}".format(str(dest)))

def copyConf(dkimConfig, dkimState):
    src = dkimState.result['signingconftemporaryfile']
    dest = dkimConfig['signingconfdestinationfile']
    log.info('  {} -> {}'.format(src, dest))
    try:
        rv = check_output(('sudo', 'mkdir', '-p', os.path.dirname(str(dest))))
    except CalledProcessError as e:
        log.error("Failed to create directory path for {}".format(str(dest)))
    try:
        rv = check_output(('sudo', 'cp', '-rfLT', str(src), str(dest)))
    except CalledProcessError as e:
        log.error("Failed to copy file")
        log.error("  {} -> {}".format(str(src), str(dest)))
    try:
        rv = check_output(('sudo', 'chown', '_rspamd:_rspamd', str(dest)))
    except CalledProcessError as e:
        log.error("Failed to change ownership of {}".format(str(dest)))

def createSelector(keybasename):
    return str(keybasename) + '_{:10d}'.format(int(time.time()))

def createDKIM(keysize, destination):
    makeDir(os.path.dirname(str(destination)))
    try:
        keyTxt = str(check_output(('sudo', 'rspamadm', 'dkim_keygen', '-b', str(int(keysize)), '-s', str('dkimkey'), '-k', str(destination))))
    except CalledProcessError as e:
        log.error(e.output)
        raise(e)
    v = keyTxt.split('v=')[1].split(';')[0]
    k = keyTxt.split('k=')[1].split(';')[0]
    pL = keyTxt.split('p=')[1].split('\"')
    p = "".join([e for e in pL if ")" not in e and "\n" not in e])
    p = p.replace("\\n", "").replace("\\t", "")
    return {'v': v, 'k': k, 'p': p, 'keyfile': str(destination)}


def createConf(signingConfTemplateFile, signingConfDestFile, keyFile, keySelector):
    f = open(os.path.expanduser(signingConfTemplateFile), 'r')
    templateContent = f.read()
    f.close()
    template = Template(templateContent)
    confDestContent = str(template.render(keyselector=keySelector, keyfile=keyFile))
    makeDir(os.path.dirname(str(signingConfDestFile)))
    f = open(os.path.expanduser(signingConfDestFile), 'w')
    f.write(confDestContent)
    f.close()
    return {'signingconftemporaryfile': signingConfDestFile, 'keyname': keySelector}


